<?php

namespace Drupal\rules_repeated_events_test\Tests;

/**
 * Tests per-user rules.
 */
class DailyEventsTestCase extends RepeatedEventsTestBase {

  public static function getInfo() {
    return static::getInfoDefaults() + array(
      'name' => 'Daily Events tests',
      'description' => 'Tests rules on daily events.',
    );
  }

  /**
   * Tests daily event scheduled on current day.
   */
  function testDailyEventScheduledOnCurrentDay() {
    $rule = rules_reaction_rule();
    $oneMinuteLater = strtotime('+1 minute');

    // Create daily events rule, with the scheduled time (in the future) like 8:15pm.
    $scheduledTime = date('g', $oneMinuteLater) . ':' . date('i', $oneMinuteLater) . date('a', $oneMinuteLater);
    $rule->event('daily_events', array('time' => $scheduledTime))
      ->action('repeated_events_simple_action');
    $rule->integrityCheck()->save('daily_events_rule');

    // Test rule creation.
    $results = entity_load('rules_config', array('daily_events_rule'));
    $this->assertEqual(1, count($results), 'Daily events rule is created successfully.');

    // Start the event watcher by clear the event cache.
    \RulesEventSet::rebuildEventCache();

    // Test watching events.
    $handler = rules_get_event_handler('daily_events', array('time' => $scheduledTime));
    $this->assertTrue($handler->isWatching(), "The daily events rule handler is watching.");

    // Test simple action is scheduled on current day.
    $result = db_select('rules_scheduler', 'rs')
      ->fields('rs', array('date'))
      ->condition('identifier', "{$handler->getEventName()}--{$handler->getEventNameSuffix()}")
      ->execute()
      ->fetchColumn();
    $scheduledTimeStamp = strtotime($scheduledTime);
    $this->assertEqual($scheduledTimeStamp, $result, "The simple action will be trigger one minute latter.");

    // Run cron to let the rules scheduler do its work.
    drupal_cron_run();

    // Make sure action is not executed.
    $this->assertFalse(\SimpleVariable::isExecuted(), "Simple action is not executed.");
  }

  /**
   * Tests daily event scheduled on next day.
   */
  function testDailyEventScheduledOnNextDay() {
    $rule = rules_reaction_rule();
    $oneMinuteBefore = strtotime('-1 minute');

    // Create daily events rule, with the scheduled time (in the future) like 8:15pm.
    $scheduledTime = date('g', $oneMinuteBefore) . ':' . date('i', $oneMinuteBefore) . date('a', $oneMinuteBefore);
    $rule->event('daily_events', array('time' => $scheduledTime))
      ->action('repeated_events_simple_action');
    $rule->integrityCheck()->save('daily_events_rule');

    // Test rule creation.
    $results = entity_load('rules_config', array('daily_events_rule'));
    $this->assertEqual(1, count($results), 'Daily events rule is created successfully.');

    // Start the event watcher by clear the event cache.
    \RulesEventSet::rebuildEventCache();

    // Test watching events.
    $handler = rules_get_event_handler('daily_events', array('time' => $scheduledTime));
    $this->assertTrue($handler->isWatching(), "The daily events rule handler is watching.");

    // Test simple action is scheduled on next day.
    $result = db_select('rules_scheduler', 'rs')
      ->fields('rs', array('date'))
      ->condition('identifier', "{$handler->getEventName()}--{$handler->getEventNameSuffix()}")
      ->execute()
      ->fetchColumn();
    $scheduledTimeStamp = strtotime($scheduledTime);
    $scheduledTimeStamp = strtotime('+1 day', $scheduledTimeStamp);
    $this->assertEqual($scheduledTimeStamp, $result, "The simple action will be trigger on next day.");

    // Run cron to let the rules scheduler do its work.
    drupal_cron_run();

    // Make sure action is not executed.
    $this->assertFalse(\SimpleVariable::isExecuted(), "Simple action is not executed.");
  }

  /**
   * Tests daily event re-scheduled.
   */
  function testDailyEventRescheduled() {
    $rule = rules_reaction_rule();
    $oneMinuteLater = strtotime('+1 minute');

    // Create daily events rule, with the scheduled time (in the future) like 8:15pm.
    $scheduledTime = date('g', $oneMinuteLater) . ':' . date('i', $oneMinuteLater) . date('a', $oneMinuteLater);
    $rule->event('daily_events', array('time' => $scheduledTime))
      ->action('repeated_events_simple_action');
    $rule->integrityCheck()->save('daily_events_rule');

    // Test rule creation.
    $results = entity_load('rules_config', array('daily_events_rule'));
    $this->assertEqual(1, count($results), 'Daily events rule is created successfully.');

    // Start the event watcher by clear the event cache.
    \RulesEventSet::rebuildEventCache();

    // Test watching events.
    $handler = rules_get_event_handler('daily_events', array('time' => $scheduledTime));
    $this->assertTrue($handler->isWatching(), "The daily events rule handler is watching.");

    // The rule is scheduled on next day. We are moving it to the past.
    $scheduledTimeStamp = strtotime($scheduledTime);
    $scheduledTimeStamp = strtotime('-1 day', $scheduledTimeStamp);
    $num_updated = db_update('rules_scheduler')
      ->fields(array(
        'date' => $scheduledTimeStamp,
      ))
      ->condition('identifier', "{$handler->getEventName()}--{$handler->getEventNameSuffix()}")
      ->execute();
    $this->assertEqual(1, $num_updated, "The simple action is not executed in one day.");

    // Run cron to let the rules scheduler do its work.
    drupal_cron_run();

    // Make sure action is executed.
    $this->assertTrue(\SimpleVariable::isExecuted(), "Simple action is executed.");

    // Test simple action is scheduled on next day.
    $scheduledTimeStamp = strtotime($scheduledTime);
    $result = db_select('rules_scheduler', 'rs')
      ->fields('rs', array('date'))
      ->condition('identifier', "{$handler->getEventName()}--{$handler->getEventNameSuffix()}")
      ->execute()
      ->fetchColumn();
    $this->assertEqual($scheduledTimeStamp, $result, "The simple action will be trigger on a minute later.");
  }
}
